package cn.com.infosec.crypto.engines;

import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.Digest;
import cn.com.infosec.crypto.InvalidCipherTextException;
import cn.com.infosec.crypto.Wrapper;
import cn.com.infosec.crypto.digests.SHA1Digest;
import cn.com.infosec.crypto.modes.CBCBlockCipher;
import cn.com.infosec.crypto.params.ParametersWithIV;
import cn.com.infosec.crypto.params.ParametersWithRandom;
import java.security.SecureRandom;

public class RC2WrapEngine implements Wrapper {
   private static final byte[] IV2 = new byte[]{74, -35, -94, 44, 121, -24, 33, 5};
   private CBCBlockCipher engine;
   private CipherParameters param;
   private ParametersWithIV paramPlusIV;
   private byte[] iv;
   private boolean forWrapping;
   private SecureRandom sr;
   Digest sha1 = new SHA1Digest();
   byte[] digest = new byte[20];

   private byte[] calculateCMSKeyChecksum(byte[] key) {
      byte[] var2;
      byte[] var10000 = var2 = new byte[8];
      this.sha1.update(key, 0, key.length);
      this.sha1.doFinal(this.digest, 0);
      System.arraycopy(this.digest, 0, var2, 0, 8);
      return var10000;
   }

   private boolean checkCMSKeyChecksum(byte[] key, byte[] checksum) {
      byte[] var3 = this.calculateCMSKeyChecksum(key);
      if (checksum.length != var3.length) {
         return false;
      } else {
         for(int key1 = 0; key1 != checksum.length; ++key1) {
            if (checksum[key1] != var3[key1]) {
               return false;
            }
         }

         return true;
      }
   }

   public void init(boolean forWrapping, CipherParameters param) {
      this.forWrapping = forWrapping;
      this.engine = new CBCBlockCipher(new RC2Engine());
      if (param instanceof ParametersWithRandom) {
         ParametersWithRandom var10000 = (ParametersWithRandom)param;
         this.sr = ((ParametersWithRandom)param).getRandom();
         param = var10000.getParameters();
      } else {
         SecureRandom forWrapping1;
         forWrapping1 = new SecureRandom.<init>();
         this.sr = forWrapping1;
      }

      if (param instanceof ParametersWithIV) {
         ParametersWithIV forWrapping2;
         this.paramPlusIV = forWrapping2 = (ParametersWithIV)param;
         this.iv = forWrapping2.getIV();
         this.param = this.paramPlusIV.getParameters();
         if (!this.forWrapping) {
            throw new IllegalArgumentException("You should not supply an IV for unwrapping");
         }

         byte[] var3;
         if ((var3 = this.iv) == null || var3.length != 8) {
            throw new IllegalArgumentException("IV is not 8 octets");
         }
      } else {
         this.param = param;
         if (this.forWrapping) {
            RC2WrapEngine var8 = this;
            byte[] forWrapping3;
            this.iv = forWrapping3 = new byte[8];
            this.sr.nextBytes(forWrapping3);
            RC2WrapEngine var10003 = this;
            CipherParameters var4 = this.param;
            var8.paramPlusIV = new ParametersWithIV(var4, var10003.iv);
         }
      }

   }

   public String getAlgorithmName() {
      return "RC2";
   }

   public byte[] wrap(byte[] in, int inOff, int inLen) {
      if (!this.forWrapping) {
         throw new IllegalStateException("Not initialized for wrapping");
      } else {
         int var4;
         int var5;
         if ((var5 = (var4 = inLen + 1) % 8) != 0) {
            var5 = var4 + (8 - var5);
         } else {
            var5 = var4;
         }

         byte[] var10002 = in;
         (in = new byte[var5])[0] = (byte)inLen;
         System.arraycopy(var10002, inOff, in, 1, inLen);
         int var10000 = inOff = var5 - inLen - 1;
         byte[] inLen1 = new byte[var10000];
         if (var10000 > 0) {
            this.sr.nextBytes(inLen1);
            System.arraycopy(inLen1, 0, in, var4, inOff);
         }

         byte[] inOff1;
         var10000 = inLen = var5 + (inOff1 = this.calculateCMSKeyChecksum(in)).length;
         byte[] var12;
         byte[] var10004 = var12 = new byte[inLen];
         System.arraycopy(in, 0, var12, 0, var5);
         int in1 = inOff1.length;
         System.arraycopy(inOff1, 0, var12, var5, in1);
         in = new byte[inLen];
         System.arraycopy(var10004, 0, in, 0, inLen);
         inOff = inLen / this.engine.getBlockSize();
         if (var10000 % this.engine.getBlockSize() != 0) {
            throw new IllegalStateException("Not multiple of block length");
         } else {
            this.engine.init(true, this.paramPlusIV);

            for(var4 = 0; var4 < inOff; ++var4) {
               var5 = var4 * this.engine.getBlockSize();
               this.engine.processBlock(in, var5, in, var5);
            }

            var10000 = var5 = (var12 = this.iv).length + inLen;
            byte[] var6 = new byte[var5];
            in1 = var12.length;
            System.arraycopy(var12, 0, var6, 0, in1);
            in1 = this.iv.length;
            System.arraycopy(in, 0, var6, in1, inLen);
            byte[] var7 = new byte[var10000];

            for(in1 = 0; in1 < var5; var7[in1] = var6[var5 - ++in1]) {
            }

            ParametersWithIV in2;
            in2 = new ParametersWithIV.<init>(this.param, IV2);
            this.engine.init(true, in2);

            for(in1 = 0; in1 < inOff + 1; ++in1) {
               inLen = in1 * this.engine.getBlockSize();
               this.engine.processBlock(var7, inLen, var7, inLen);
            }

            return var7;
         }
      }
   }

   public byte[] unwrap(byte[] in, int inOff, int inLen) {
      if (this.forWrapping) {
         throw new IllegalStateException("Not set for unwrapping");
      } else if (in == null) {
         throw new InvalidCipherTextException("Null pointer as ciphertext");
      } else if (inLen % this.engine.getBlockSize() != 0) {
         throw new InvalidCipherTextException("Ciphertext not multiple of " + this.engine.getBlockSize());
      } else {
         ParametersWithIV in1;
         in1 = new ParametersWithIV.<init>(this.param, IV2);
         this.engine.init(false, in1);
         System.arraycopy(in, inOff, in = new byte[inLen], 0, inLen);

         for(inOff = 0; inOff < inLen / this.engine.getBlockSize(); ++inOff) {
            int var4 = inOff * this.engine.getBlockSize();
            this.engine.processBlock(in, var4, in, var4);
         }

         byte[] var13 = new byte[inLen];

         for(inOff = 0; inOff < inLen; var13[inOff] = in[inLen - ++inOff]) {
         }

         this.iv = in = new byte[8];
         int var5;
         byte[] inOff1;
         byte[] var10000 = inOff1 = new byte[var5 = inLen - 8];
         System.arraycopy(var13, 0, in, 0, 8);
         System.arraycopy(var13, 8, inOff1, 0, var5);
         ParametersWithIV var10004 = in1 = new ParametersWithIV;
         CipherParameters inOff2 = this.param;
         var10004.<init>(inOff2, this.iv);
         this.paramPlusIV = var10004;
         this.engine.init(false, in1);
         byte[] var6 = new byte[var5];
         System.arraycopy(var10000, 0, var6, 0, var5);

         for(int in2 = 0; in2 < var5 / this.engine.getBlockSize(); ++in2) {
            inOff = in2 * this.engine.getBlockSize();
            this.engine.processBlock(var6, inOff, var6, inOff);
         }

         int var7;
         in = new byte[var7 = var5 - 8];
         inOff1 = new byte[8];
         System.arraycopy(var6, 0, in, 0, var7);
         System.arraycopy(var6, var7, inOff1, 0, 8);
         if (this.checkCMSKeyChecksum(in, inOff1)) {
            if (var7 - ((in[0] & 255) + 1) <= 7) {
               byte var8;
               var10000 = inOff1 = new byte[var8 = in[0]];
               System.arraycopy(in, 1, inOff1, 0, var8);
               return var10000;
            } else {
               StringBuilder inOff3;
               StringBuilder var10002 = inOff3 = new StringBuilder;
               inOff3.<init>("too many pad bytes (");
               throw new InvalidCipherTextException(var10002.append(var7 - ((in[0] & 255) + 1)).append(")").toString());
            }
         } else {
            throw new InvalidCipherTextException("Checksum inside ciphertext is corrupted");
         }
      }
   }
}
